diff --git a/pgadmin/frm/frmMaintenance.cpp b/pgadmin/frm/frmMaintenance.cpp
index d044b12..6e7e695 100644
--- a/pgadmin/frm/frmMaintenance.cpp
+++ b/pgadmin/frm/frmMaintenance.cpp
@@ -90,6 +90,9 @@ wxString frmMaintenance::GetHelpPage() const
 		case 2:
 			page = wxT("pg/sql-reindex");
 			break;
+		case 3:
+			page = wxT("pg/sql-cluster");
+			break;
 	}
 	return page;
 }
@@ -104,11 +107,17 @@ void frmMaintenance::OnAction(wxCommandEvent &ev)
 	chkAnalyze->Enable(isVacuum);
 
 	bool isReindex = (rbxAction->GetSelection() == 2);
-	if (isReindex)
+	bool isCluster = (rbxAction->GetSelection() == 3);
+	if (isReindex || (isCluster && !conn->BackendMinimumVersion(8, 4)))
 	{
 		chkVerbose->SetValue(false);
+		chkVerbose->Enable(false);
+	}
+	else
+	{
+		chkVerbose->SetValue(true);
+		chkVerbose->Enable(true);
 	}
-	chkVerbose->Enable(!isReindex);
 }
 
 
@@ -168,6 +177,28 @@ wxString frmMaintenance::GetSql()
 			}
 			break;
 		}
+		case 3:
+		{
+			sql = wxT("CLUSTER ");
+
+			if (chkVerbose->GetValue())
+			        sql += wxT("VERBOSE ");
+			if (object->GetMetaType() == PGM_TABLE)
+				sql += object->GetQuotedFullIdentifier();
+			if (object->GetMetaType() == PGM_INDEX || object->GetMetaType() == PGM_UNIQUE
+			    || object->GetMetaType() == PGM_PRIMARYKEY)
+			{
+				sql += object->GetTable()->GetQuotedFullIdentifier();
+				if (conn->BackendMinimumVersion(8, 4))
+				{
+					sql += wxT(" USING ") + object->GetQuotedIdentifier();
+				}
+				else
+				{
+					sql += wxT(" ON ") + object->GetQuotedIdentifier();
+				}
+			}
+		}
 	}
 
 	return sql;
diff --git a/pgadmin/ui/frmMaintenance.xrc b/pgadmin/ui/frmMaintenance.xrc
index 436c0d4..ab8fc49 100644
--- a/pgadmin/ui/frmMaintenance.xrc
+++ b/pgadmin/ui/frmMaintenance.xrc
@@ -28,6 +28,7 @@
                       <item>VACUUM</item>
                       <item>ANALYZE</item>
                       <item>REINDEX</item>
+                      <item>CLUSTER</item>
                     </content>
                     <selection>0</selection>
                     <style>wxRA_SPECIFY_ROWS</style>
